﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace IRaCIS.Core.Application.Service.Reading.Dto
{

    public class TumorEvaluationInfo
    {


        public List<ExportTumorEvaluationInfo> VisitList { get; set; }
    }

    public class ExportTumorEvaluationInfo
    {
        /// <summary>
        /// 所有访视的靶病灶列表
        /// </summary>
        public List<ExportVisitData> VisitList { get; set; }

        public bool IsLastPage { get; set; } = false;

        /// <summary>
        /// 受试者Code
        /// </summary>
        public string SubjectCode { get; set; } =string.Empty;

        public string SubjectName { get; set; } = string.Empty;

        public string ResearchProgramNo { get; set; } = string.Empty;

        public ExportData Lesion { get; set; }=new ExportData();

        /// <summary>
        /// 访视一
        /// </summary>
        public ExportVisitData VisitOne
        {
            get
            {
                return VisitList[0];
            }
        }

        /// <summary>
        /// 访视二
        /// </summary>
        public ExportVisitData VisitTwo
        {
            get
            {
                if (VisitList.Count >= 2)
                {
                    return VisitList[1];
                }
                else
                {
                    return new ExportVisitData();
                }
              
            }
        }
        /// <summary>
        /// 访视三
        /// </summary>
        public ExportVisitData VisitThree
        {
            get
            {
                if (VisitList.Count >= 3)
                {
                    return VisitList[2];
                }
                else
                {
                    return new ExportVisitData();
                }

            }
        }
        /// <summary>
        /// 靶病灶信息
        /// </summary>
        public List<TargetInfo> TargetInfoList
        {
            get
            {
                List<TargetInfo> targets=new List<TargetInfo>();

                var index = 0;
                foreach (var item in VisitOne.VisitTargetInfoList)
                {
                    TargetInfo targetInfo = new TargetInfo()
                    {
                        Title = item.Title,
                        VisitOneImgNum = item.VisitImgNum,
                        VisitOneLength = item.VisitLength.ToString(),
                        VisitTwoImgNum = VisitList.Count >= 2 ? VisitTwo.VisitTargetInfoList[index].VisitImgNum : string.Empty,
                        VisitTwoLength = VisitList.Count >= 2 ? VisitTwo.VisitTargetInfoList[index].VisitLength.ToString() : string.Empty,
                        VisitThreeImgNum = VisitList.Count >= 3 ? VisitThree.VisitTargetInfoList[index].VisitImgNum : string.Empty,
                        VisitThreeLength = VisitList.Count >= 3 ? VisitThree.VisitTargetInfoList[index].VisitLength.ToString() : string.Empty
                    };
                    targets.Add(targetInfo);
                    index++;
                }
                return targets;

            }
        }

        /// <summary>
        /// 非靶病灶信息
        /// </summary>
        public List<TargetInfo> NoTargetInfoList
        {
            get
            {
                List<TargetInfo> targets = new List<TargetInfo>();

                var index = 0;
                foreach (var item in VisitOne.VisitNoTargetInfoList)
                {
                    TargetInfo targetInfo = new TargetInfo()
                    {
                        Title = item.Title,
                        VisitOneImgNum = item.VisitImgNum,
                        VisitOneLength = item.VisitLength!=null? item.VisitLength.Value.ToString():string.Empty,
                        VisitTwoImgNum = VisitList.Count >= 2 ? VisitTwo.VisitNoTargetInfoList[index].VisitImgNum : string.Empty,
                        VisitTwoLength = VisitList.Count >= 2&& VisitTwo.VisitNoTargetInfoList[index].VisitLength!=null ? VisitTwo.VisitNoTargetInfoList[index].VisitLength.Value.ToString() : string.Empty,
                        VisitThreeImgNum = VisitList.Count >= 3 ? VisitThree.VisitNoTargetInfoList[index].VisitImgNum : string.Empty,
                        VisitThreeLength = VisitList.Count >= 3&& VisitThree.VisitNoTargetInfoList[index].VisitLength!=null ? VisitThree.VisitNoTargetInfoList[index].VisitLength.Value.ToString() : string.Empty
                    };
                    targets.Add(targetInfo);
                    index++;
                }
                return targets;

            }
        }
    }

 

    public class TargetInfo
    {
        public string Title { get; set; } = string.Empty;

        public string VisitOneImgNum { get; set; } = string.Empty;

        public string VisitOneLength { get; set; } = string.Empty;

        public string VisitTwoImgNum { get; set; } = string.Empty;

        public string VisitTwoLength { get; set; } = string.Empty;

        public string VisitThreeImgNum { get; set; } = string.Empty;

        public string VisitThreeLength { get; set; } = string.Empty;
    }

    public class ExportData
    {
        public string OneVisitName { get; set; } = string.Empty;

        public string OneCheckDate { get; set; } = string.Empty;

        public string OneCheckInfoStr { get; set; } = string.Empty;

        public string OneTargetSum { get; set; } = string.Empty;

        public string OneHaveNewLesion { get; set; } = string.Empty;

        public string OneTargetResult { get; set; } = string.Empty;

        public string OneNoTargetResult { get; set; } = string.Empty;

        public string OneOverallResult { get; set; } = string.Empty;

        public string OneDoctorSignTime { get; set; } = string.Empty;

        public string TwoVisitName { get; set; } = string.Empty;

        public string TwoCheckDate { get; set; } = string.Empty;


        public string TwoCheckInfoStr { get; set; } = string.Empty;

        public string TwoTargetSum { get; set; } = string.Empty;

        public string TwoHaveNewLesion { get; set; } = string.Empty;

        public string TwoTargetResult { get; set; } = string.Empty;

        public string TwoNoTargetResult { get; set; } = string.Empty;

        public string TwoOverallResult { get; set; } = string.Empty;

        public string TwoDoctorSignTime { get; set; } = string.Empty;

        public string ThreeVisitName { get; set; } = string.Empty;

        public string ThreeCheckDate { get; set; } = string.Empty;

        public string ThreeCheckInfoStr { get; set; } = string.Empty;

        public string ThreeTargetSum { get; set; } = string.Empty;

        public string ThreeHaveNewLesion { get; set; } = string.Empty;

        public string ThreeTargetResult { get; set; } = string.Empty;

        public string ThreeNoTargetResult { get; set; } = string.Empty;

        public string ThreeOverallResult { get; set; } = string.Empty;

        public string ThreeDoctorSignTime { get; set; } = string.Empty;
    }

    public class ExportVisitData
    {
        /// <summary>
        /// 访视名称
        /// </summary>
        public string VisitName { get; set; } = string.Empty;

        /// <summary>
        /// 访视序号
        /// </summary>
        public decimal VisitNum { get; set; }

        /// <summary>
        /// 检查日期
        /// </summary>
        public string CheckDate { get; set; } = string.Empty;

        /// <summary>
        /// 检查信息
        /// </summary>
        public List<DicomStudyInfo> CheckInfoList { get; set; } = new List<DicomStudyInfo>();

        /// <summary>
        /// 检查信息字符串
        /// </summary>
        public string CheckInfoStr => string.Join("；", CheckInfoList.Select(x => $"{x.Modalities}/{x.BodyPartExamined}"));

        /// <summary>
        /// 是否有新病灶
        /// </summary>
        public string HaveNewLesion { get; set; } = string.Empty;

        /// <summary>
        /// 靶病灶结果
        /// </summary>
        public string TargetResult { get; set; } = string.Empty;

        /// <summary>
        /// 非靶病灶结果
        /// </summary>
        public string NoTargetResult { get; set; } = string.Empty;

        /// <summary>
        /// 总体结果
        /// </summary>
        public string OverallResult { get; set; } = string.Empty;

        /// <summary>
        /// 医生名称
        /// </summary>
        public string DoctorName { get; set; } = string.Empty;

        /// <summary>
        /// 签名时间
        /// </summary>
        public string SignTime { get; set; } = string.Empty;

        /// <summary>
        /// 医生签名时间
        /// </summary>
        public string DoctorSignTime
        {
            get
            {
                return DoctorName==string.Empty?string.Empty: $"{DoctorName}/{SignTime}";
            }
        }

        /// <summary>
        /// 靶病灶信息
        /// </summary>
        public List<VisitLesionInfo> VisitTargetInfoList { get; set; } = new List<VisitLesionInfo>();

        /// <summary>
        /// 靶病灶总长度
        /// </summary>
        public decimal? TargetSum
        {
            get
            {
                return VisitTargetInfoList.Sum(x => x.VisitLength);
            }
        }


        /// <summary>
        /// 非靶病灶信息
        /// </summary>
        public List<VisitLesionInfo> VisitNoTargetInfoList { get; set; } = new List<VisitLesionInfo>();

    }

    public class VisitLesionInfo
    {
        /// <summary>
        /// 序号
        /// </summary>
        public string RowMark { get; set; } = string.Empty;

        /// <summary>
        /// 器官
        /// </summary>
        public string Organ { get; set; } = string.Empty;

        /// <summary>
        /// 部位
        /// </summary>
        public string Part { get; set; } = string.Empty;

        public int Number { get; set; }

        public string Title {
            get
            {
                if (RowMark != string.Empty)
                {
                    return $"{Number}：{RowMark}，{Organ}：{Part}";
                }
                else
                {
                    return $"{Number}";
                }

            } 
        }

        /// <summary>
        /// 检查号
        /// </summary>
        public int StudyCode { get; set; }

        public Guid? RowId { get; set; }

        /// <summary>
        /// 序列号
        /// </summary>
        public int SeriesNumber { get; set; }

        /// <summary>
        /// 图像号
        /// </summary>
        public int InstanceNumber { get; set; }

        /// <summary>
        /// 图像号
        /// </summary>
        public string VisitImgNum 
        {
            get
            {
                if (RowId == null)
                {
                    return string.Empty;
                }
                else
                {
                    return $"{StudyCode}/{SeriesNumber}/{InstanceNumber}";
                }
                  
            }
        }

        /// <summary>
        /// 长度
        /// </summary>
        public decimal? VisitLength { get; set; }
    }



    public class DicomStudyInfo
    {
        /// <summary>
        /// 检查类型
        /// </summary>
        public string Modalities { get; set; } = string.Empty;

        /// <summary>
        /// 标记
        /// </summary>
        public string BodyPartExamined { get; set; }
    }
}
